// Original Ngaro Virtual Machine and Uki framework:
//	Copyright (C) 2008, 2009, 2010 Charles Childers
// Go port
//	Copyright 2009, 2010 JGL
// Public Domain or LICENSE file

package main

import (
	"./ngaro"
	"flag"
	"fmt"
	"os"
)

var Usage = func() {
	fmt.Fprint(os.Stderr, `
Gonga is the Go version of the Ngaro virtual machine.

Usage:
	gonga [options] [image file] [-w input file(s)]

It will run the image file specified in the command line.
By default, gongaImage will be loaded, retroImage if that
fails. Input files can be added preceeding the file name
with -w. They will be read in inverse order (LIFO).

Options:
`)
	flag.PrintDefaults()
}

var shrink = flag.Bool("shrink", false, "shrink image dump file")
var size = flag.Int("s", 50000, "image size")
var dump = flag.String("d", "retro.img", "image dump file")
var retroImage = [...]int{}

func main() {
	flag.Usage = Usage
	flag.Parse()
	ngaro.ShrinkImage = *shrink
	ngaro.ClearScreen = func() { fmt.Printf("\033[2J\033[1;1H") }

	var img []int
	var err os.Error
	if flag.NArg() == 0 {
		img = &retroImage
	}
	with := false
	input := &ngaro.Input{os.Stdin, nil}
	for i := 0; i < flag.NArg(); i++ {
		if with {
			if flag.Arg(i) == "-" {
				input = &ngaro.Input{os.Stdin, input}
			} else if f, err := os.Open(flag.Arg(i), os.O_RDONLY, 0666); err != nil {
				panic("Wrong input file: ", flag.Arg(i))
			} else {
				input = &ngaro.Input{f, input}
			}
			with = false
		} else if flag.Arg(i) == "-w" || flag.Arg(i) == "--with" {
			with = true
		} else if len(img) != 0 {
			panic("Too many arguments")
		} else if img, err = ngaro.LoadDump(flag.Arg(i), *size); err != nil {
			panic("Wrong image file: ", flag.Arg(i))
		}
	}
	if len(img) == 0 {
		img, err = ngaro.LoadDump("gongaImage", *size)
		if err != nil {
			img, err = ngaro.LoadDump("retroImage", *size)
		}
		if err != nil {
			panic("Image file not found")
		}
	}
	vm := ngaro.NewVM(img, *dump, input, os.Stdout)

	if ok := <-vm.EndOk; !ok {
		os.Exit(1)
	}
}
